import * as React from 'react';
import { Annotation, Shape, Shapes } from '../../models';
import { componentType, Dimensions } from '../../services';
import { RootReducerType } from '../../store';
/**
 * Defined the component's own properties
 */
export interface OwnProps<T extends Shape> {
    shape: T;
    shapeType: keyof Shapes;
    canEdit: boolean;
    zoomRatio: number;
    additionalOffset?: Dimensions;
}
/**
 * maps state fields from the store to component props
 * @param state the redux state
 */
declare const mapStateToProps: (state: RootReducerType, ownProps: OwnProps<Shape>) => {
    zoomMode: import("../../store/Viewer").ZoomMode;
    zoomLevel: number;
};
/**
 * maps state actions from the store to component props
 * @param state the redux state
 */
declare const mapDispatchToProps: {
    updateShapeData: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string, shapeData: Shapes[K][0]) => {
        type: string;
        shapeType: K;
        shapeGuid: string;
        shapeData: Shapes[K][0];
    };
    removeShape: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string) => {
        type: string;
        shapeType: K;
        shapeGuid: string;
    };
};
/**
 * Class for displaying Shapes on a document page
 */
declare abstract class ShapeComponent<T extends Shape = Shape> extends React.Component<componentType<typeof mapStateToProps, typeof mapDispatchToProps, OwnProps<T>>> {
    /** the component state */
    state: {
        focused: boolean;
        onResized: any;
        onBlur: any;
        onDragStart: any;
        handleKeyPress: any;
        onFocus: any;
    };
    /** The type of the shape instance */
    protected abstract shapeType: keyof Shapes;
    /**
     * Method that returns the shape dimensions as CSS properties
     * @param shape the shape instance
     * @param offsetX optional X offset
     * @param offsetY optional Y offset
     */
    protected getShapeDimensions(shape: Shape, offsetX?: number, offsetY?: number): React.CSSProperties;
    /** event that will be triggered on resize */
    protected onResized(ev: React.MouseEvent<HTMLElement>): void;
    /** onDragStart event handler for the Shape instance */
    protected onDragStart(ev: React.DragEvent<HTMLElement>): void;
    /** abstract method for shape rendering */
    abstract renderShape(): JSX.Element;
    /** onKeyPress event handler for deleting shapes */
    protected handleKeyPress(ev: React.KeyboardEvent<HTMLDivElement>): void;
    /** onFocus event handler that updates the 'focused' property */
    onFocus(ev: React.FocusEvent<HTMLDivElement>): void;
    /** onBlur event handler that updates the 'focused' property */
    onBlur(ev: React.FocusEvent<HTMLDivElement>): void;
    /**
     * renders the component
     */
    render(): JSX.Element;
}
declare class ShapeRedaction extends ShapeComponent {
    /** type of the Shape */
    readonly shapeType: string;
    /** renders the Shape component */
    renderShape(): JSX.Element;
}
declare const shapeRedaction: import("react-redux").ConnectedComponentClass<typeof ShapeRedaction, Pick<{
    zoomMode: import("../../store/Viewer").ZoomMode;
    zoomLevel: number;
} & {
    updateShapeData: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string, shapeData: Shapes[K][0]) => {
        type: string;
        shapeType: K;
        shapeGuid: string;
        shapeData: Shapes[K][0];
    };
    removeShape: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string) => {
        type: string;
        shapeType: K;
        shapeGuid: string;
    };
} & OwnProps<Shape>, "zoomRatio" | "canEdit" | "shape" | "shapeType" | "additionalOffset"> & OwnProps<Shape>>;
declare class ShapeAnnotation extends ShapeComponent<Annotation> {
    /** type of the Shape */
    readonly shapeType: string;
    /** keypress handler for to stop event bubbling and shape deletion */
    protected handleKeyPress(ev: React.KeyboardEvent<HTMLDivElement>): void;
    /** onBlur handler for updating inner text */
    onBlur(ev: React.FocusEvent<HTMLDivElement>): void;
    /** renders the Shape component */
    renderShape(): JSX.Element;
}
declare const shapeAnnotation: import("react-redux").ConnectedComponentClass<typeof ShapeAnnotation, Pick<{
    zoomMode: import("../../store/Viewer").ZoomMode;
    zoomLevel: number;
} & {
    updateShapeData: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string, shapeData: Shapes[K][0]) => {
        type: string;
        shapeType: K;
        shapeGuid: string;
        shapeData: Shapes[K][0];
    };
    removeShape: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string) => {
        type: string;
        shapeType: K;
        shapeGuid: string;
    };
} & OwnProps<Annotation>, "zoomRatio" | "canEdit" | "shape" | "shapeType" | "additionalOffset"> & OwnProps<Shape>>;
declare class ShapeHighlight extends ShapeComponent {
    /** type of the Shape */
    readonly shapeType: string;
    /** renders the Highlight shape component */
    renderShape(): JSX.Element;
}
declare const shapeHighlight: import("react-redux").ConnectedComponentClass<typeof ShapeHighlight, Pick<{
    zoomMode: import("../../store/Viewer").ZoomMode;
    zoomLevel: number;
} & {
    updateShapeData: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string, shapeData: Shapes[K][0]) => {
        type: string;
        shapeType: K;
        shapeGuid: string;
        shapeData: Shapes[K][0];
    };
    removeShape: <K extends "redactions" | "highlights" | "annotations">(shapeType: K, shapeGuid: string) => {
        type: string;
        shapeType: K;
        shapeGuid: string;
    };
} & OwnProps<Shape>, "zoomRatio" | "canEdit" | "shape" | "shapeType" | "additionalOffset"> & OwnProps<Shape>>;
export { shapeAnnotation as ShapeAnnotation, shapeHighlight as ShapeHighlight, shapeRedaction as ShapeRedaction };
